package fr.orsys.tp3.dao.impl;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.sql.DataSource;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.support.JdbcDaoSupport;
import org.springframework.stereotype.Repository;

import fr.orsys.tp3.dao.IEmployeDao;
import fr.orsys.tp3.domain.Employe;

@Repository("employeDao")
public class EmployeDao3 extends JdbcDaoSupport implements IEmployeDao {
	
	private static final Logger logger = Logger.getLogger(EmployeDao3.class);
	
	@Autowired
	public EmployeDao3(DataSource dataSource) {
		super.setDataSource(dataSource);
	}
	
	@PostConstruct
	public void onInit() {
		logger.info("The bean EmployeDao3 has been created!");
	}
	
	@PreDestroy
	public void onDestroy() {
		logger.info("The bean EmployeDao3 will be destroyed!");
	}

	@Override
	public Employe getEmployeById(int id) {
		final String EMPLOYE_SELECT = "select * from employe where id = ?";
		return getJdbcTemplate().queryForObject(EMPLOYE_SELECT, new Object[] { id }, EmployeDao3::createEmploye);
	}

	@Override
	public Employe getEmployeByLogin(String login) {
		final String EMPLOYE_SELECT = "select  * from employe where login = ?";
		return getJdbcTemplate().queryForObject(EMPLOYE_SELECT, new Object[] { login }, EmployeDao3::createEmploye);
	}

	@Override
	public void saveEmploye(Employe employe) {
		final String EMPLOYE_INSERT = "insert into employe (id, login, password, prenom, nom, email, role) values (?,?,?,?,?,?,?)";
		getJdbcTemplate().update(EMPLOYE_INSERT, new Object[] { employe.getId(), employe.getLogin(), employe.getPassword(), employe.getPrenom(), employe.getNom(), employe.getEmail(), employe.getRole() });
	}

	@Override
	public List<Employe> getAllEmployes() {
		final String EMPLOYE_SELECT = "select * from employe";
		return getJdbcTemplate().query(EMPLOYE_SELECT, EmployeDao3::createEmploye);
	}

	@Override
	public int getEmployesCount() {
		final String EMPLOYE_SELECT = "select count(*) from employe";
		return getJdbcTemplate().queryForObject(EMPLOYE_SELECT, new Object[] {}, Integer.class);
	}

	private static Employe createEmploye(ResultSet rs, int rowNum) throws SQLException {
		if (rs == null)
			return null;

		Employe emp = new Employe();
		emp.setEmail(rs.getString("email"));
		emp.setLogin(rs.getString("login"));
		emp.setNom(rs.getString("nom"));
		emp.setPrenom(rs.getString("prenom"));
		emp.setRole(rs.getString("role"));
		emp.setId(rs.getInt("id"));

		return emp;
	}

}
